@@ -1,5 +1,45 @@
 # -*- coding: utf-8 -*-
-"""Examples and demos."""
+"""
+Demos and examples (:mod:`qit.examples`)
+========================================
+
+.. currentmodule:: qit.examples
+
+This module contains various examples which demonstrate the features
+of QIT. The :func:`tour` function is a "guided tour" to the toolkit
+which runs all the demos in succession.
+
+
+Demos
+-----
+
+.. autosummary::
+
+   adiabatic_qc_3sat
+   bb84
+   grover_search
+   markov_decoherence
+   nmr_sequences
+   phase_estimation_precision
+   qft_circuit
+   quantum_channels
+   quantum_walk
+   qubit_and_resonator
+   shor_factorization
+   superdense_coding
+   teleportation
+   tour
+
+
+General-purpose quantum algorithms
+----------------------------------
+
+.. autosummary::
+
+   adiabatic_qc
+   phase_estimation
+   find_order
+"""
 # Ville Bergholm 2011-2012
 
 from __future__ import division, absolute_import, print_function, unicode_literals
@@ -177,7 +217,7 @@ def adiabatic_qc(H0, H1, s0, tmax=50):
     else:
         print('Which is not a solution!')
         if np.min(H1) > 0:
-            print('(In this problem instance there aren''t any.)')
+            print("(In this problem instance there aren't any.)")
 
 
 
@@ -343,7 +383,14 @@ def bernstein_vazirani(n=6, linear=True):
     # final Hadamards
     s = s.u_propagate(H)
 
+    plt.figure()
     s.plot()
+    title = 'Bernstein-Vazirani algorithm, '
+    if linear:
+        title += 'linear oracle'
+    else:
+        title += 'nonlinear oracle (fails)'
+    plt.title(title)
 
     p, res = s.measure()
     # measured binary vector
@@ -462,7 +509,7 @@ def markov_decoherence(T1, T2, B=None):
 
 
 
-def nmr_sequences():
+def nmr_sequences(seqs=None, titles=None):
     """NMR control sequences demo.
 
     Compares the performance of different single-qubit NMR control
@@ -472,14 +519,17 @@ def nmr_sequences():
 
     Reproduces fidelity plots in [Cummins]_.
     """
-    # Ville Bergholm 2006-2012
+    # Ville Bergholm 2006-2014
 
     from . import seq
 
     print('\n\n=== NMR control sequences for correcting systematic errors ===\n')
 
-    seqs = [seq.nmr([[pi, 0]]), seq.corpse(pi), seq.scrofulous(pi), seq.bb1(pi)]
-    titles = ['Plain $\pi$ pulse', 'CORPSE', 'SCROFULOUS', 'BB1']
+    if seqs == None:
+        seqs = [seq.nmr([[pi, 0]]), seq.corpse(pi), seq.scrofulous(pi), seq.bb1(pi)]
+        titles = ['Plain $\pi$ pulse', 'CORPSE', 'SCROFULOUS', 'BB1']
+    elif titles == None:
+        titles = ['User-given seq'] * len(seqs)
 
     psi = state('0') # initial state
     f = arange(-1, 1, 0.05)
@@ -494,27 +544,27 @@ def nmr_sequences():
         plot.state_trajectory(out, ax = ax)
 
 
-    for k, s in enumerate(seqs):
+    for tt, ss in zip(titles, seqs):
         fig = plt.figure()
         gs = GridSpec(2, 2)
-        U = seq.seq2prop(s) # target propagator
+        U = seq.seq2prop(ss) # target propagator
 
         # The two systematic error types we are interested here can be
         # incorporated into the control sequence.
         #==================================================
-        s_error = deepcopy(s)
-        s_error['A'] = s['A'] + 0.1 * 0.5j * sz  # off-resonance error (constant \sigma_z drift term)
+        s_error = deepcopy(ss)
+        s_error['A'] = ss['A'] + 0.1 * 0.5j * sz  # off-resonance error (constant \sigma_z drift term)
         ax = fig.add_subplot(gs[0, 0], projection = '3d')
-        helper(ax, s_error, titles[k] + ' evolution, off-resonance error')
+        helper(ax, s_error, tt + ' evolution, off-resonance error')
 
         #==================================================
-        s_error = deepcopy(s)
-        s_error['tau'] = s['tau'] * 1.1  # proportional pulse length error
+        s_error = deepcopy(ss)
+        s_error['tau'] = ss['tau'] * 1.1  # proportional pulse length error
         ax = fig.add_subplot(gs[1, 0], projection = '3d')
-        helper(ax, s_error, titles[k] + ' evolution, pulse length error')
+        helper(ax, s_error, tt + ' evolution, pulse length error')
 
         #==================================================
-        s_error = deepcopy(s)
+        s_error = deepcopy(ss)
         fid = empty((ng, nf))
 
         def u_fidelity(a, b):
@@ -522,9 +572,9 @@ def nmr_sequences():
             return 0.5 * abs(trace(dot(a.conj().transpose(), b)))
 
         for u in range(nf):
-            s_error['A'] = s['A'] + f[u] * 0.5j * sz  # off-resonance error
+            s_error['A'] = ss['A'] + f[u] * 0.5j * sz  # off-resonance error
             for v in range(ng):
-                s_error['tau'] = s['tau'] * (1 + g[v])  # proportional pulse length error
+                s_error['tau'] = ss['tau'] * (1 + g[v])  # proportional pulse length error
                 fid[v, u] = u_fidelity(U, seq.seq2prop(s_error))
 
         ax = fig.add_subplot(gs[:, 1])
@@ -533,7 +583,7 @@ def nmr_sequences():
         #ax.surf(X, Y, 1-fid)
         ax.set_xlabel('Off-resonance error')
         ax.set_ylabel('Pulse length error')
-        ax.set_title(titles[k] + ' fidelity')
+        ax.set_title(tt + ' fidelity')
         plt.show()
 
 
@@ -721,6 +771,7 @@ def quantum_channels(p=0.3):
         ax.set_title(T)
 
     fig = plt.figure()
+    plt.title('Quantum channels')
     n = len(channels)
     gs = GridSpec(2, int(ceil(n / 2)))
     for k in range(n):
@@ -1278,21 +1329,21 @@ def tour():
     """Guided tour to the quantum information toolkit.
 
     """
-    # Ville Bergholm 2009-2011
+    # Ville Bergholm 2009-2014
 
     print('This is the guided tour for the Quantum Information Toolkit.')
+    print("It should be run in the interactive mode, 'ipython --pylab'.")
     print('Between examples, press any key to proceed to the next one.')
 
     def pause():
         plt.waitforbuttonpress()
 
-    pause()
-
+    temp = plt.figure()
     teleportation()
     pause()
-
     superdense_coding()
     pause()
+    plt.close(temp)
 
     adiabatic_qc_3sat(5, 25)
     pause()
