@@ -1,5 +1,78 @@
 # -*- coding: utf-8 -*-
-"""Quantum states.
+"""
+Quantum states (:mod:`qit.state`)
+=================================
+
+
+In QIT, quantum states are represented by the :class:`state` class,
+defined in this module.
+
+
+.. currentmodule:: qit.state.state
+
+
+Utilities
+---------
+
+.. autosummary::
+
+   check
+   subsystems
+   dims
+   clean_selection
+   invert_selection
+   fix_phase
+   normalize
+   to_ket
+   to_op
+   trace
+   ptrace
+   ptranspose
+   reorder
+   tensor
+   plot
+
+
+Physics
+-------
+
+.. autosummary::
+
+   ev
+   var
+   prob
+   projector
+   u_propagate
+   propagate
+   kraus_propagate
+   measure
+   
+
+Quantum information
+-------------------
+
+.. autosummary::
+
+   fidelity
+   trace_dist
+   purity
+   schmidt
+   entropy
+   concurrence
+   negativity
+   lognegativity
+   scott
+   locc_convertible
+
+
+Other state representations
+---------------------------
+
+.. autosummary::
+
+   bloch_vector
+   bloch_state
+
 
 .. [NC] M.A. Nielsen, I.L. Chuang, "Quantum Computation and Quantum Information" (2000).
 """
@@ -13,25 +86,21 @@ import types
 from copy import deepcopy
 
 import numpy as np
-from numpy import array, asarray, diag, sort, prod, cumsum, cumprod, exp, sqrt, trace, dot, vdot, roll, zeros, ones, r_, kron, isscalar, nonzero, ix_, linspace, meshgrid
+from numpy import array, asarray, empty, diag, sort, prod, cumsum, cumprod, exp, sqrt, trace, dot, vdot, roll, zeros, ones, r_, kron, isscalar, nonzero, ix_, linspace, meshgrid
 from numpy.random import rand, randn
 import scipy as sp
-from scipy.linalg import norm, eigh
+from scipy.linalg import norm, eigh, eigvalsh
 from scipy.integrate import ode
 
 from .base import Q_Bell, tol
 from .lmap import *
 from .utils import *
+from .utils import _warn
 
 __all__ = ['equal_dims', 'index_muls', 'state', 'fidelity', 'trace_dist']
 
 
 
-def _warn(s):
-    """Prints a warning."""
-    print('Warning: ' + s)
-
-
 def equal_dims(s, t):
     """True if s and t have equal dimensions."""
     return s.dims() == t.dims()
@@ -39,8 +108,7 @@ def equal_dims(s, t):
 
 def index_muls(dim):
     """Index multipliers for C-ordered data"""
-    # TODO with numpy 1.6:
-    # ind = ravel_multi_index(s, dim) == dot(index_muls(dim), s)
+    # ravel_multi_index(s, dim) == dot(index_muls(dim), s)
     if len(dim) == 0:
         return array(())
     muls = roll(cumprod(dim[::-1]), 1)[::-1]
@@ -126,9 +194,8 @@ class state(lmap):
                     s = deepcopy(Q_Bell[:, ord(name[-1]) - ord('1')])
                 elif name == 'ghz':
                     # Greenberger-Horne-Zeilinger state
-                    muls = index_muls(dim)
                     for k in range(dmin):
-                        s[dot(k * ones(n), muls)] = 1
+                        s[np.ravel_multi_index(n*(k,), dim)] = 1
                 elif name == 'w':
                     # W state
                     ind = 1
@@ -150,10 +217,8 @@ class state(lmap):
                 s = numstr_to_array(s)
                 if any(s >= dim):
                     raise ValueError('Invalid basis ket.')
-                # TODO numpy 1.6:
-                # ind = ravel_multi_index(s, dim)
-                muls = index_muls(dim)
-                ind = dot(muls, s)
+
+                ind = np.ravel_multi_index(s, dim)
                 s = zeros(prod(dim)) # ket
                 s[ind] = 1
 
@@ -194,6 +259,30 @@ class state(lmap):
 
 
 # utility methods
+# TODO design issue: for valid states, lots of these funcs should return reals (marked with a commented-out .real). should we just drop the imaginary part? what about if the state is invalid, how will the user know? what about numerical errors?
+# TODO same thing except with normalization, should we assume states are normalized?
+
+    def check(self):
+        """Checks the validity of the state.
+
+        Makes sure it is normalized, and if an operator, Hermitian and semipositive.
+        """
+        ok = True
+        if abs(self.trace() - 1) > tol:
+            _warn('State not properly normalized.')
+            ok = False
+
+        if not self.is_ket():
+            if norm(self.data - self.data.conj().transpose()) > tol:
+                _warn('State operator not Hermitian.')
+                ok = False
+            if min(eigvalsh(self.data)) < -tol:
+                _warn('State operator not semipositive.')
+                ok = False
+
+        if not ok:
+            raise ValueError('Not a valid state.')
+
 
     def subsystems(self):
         """Number of subsystems in the state."""
@@ -239,7 +328,7 @@ class state(lmap):
 
 
     def normalize(self, inplace=False):
-        """Normalize the state."""
+        """Normalize the state to unity."""
         s = self._inplacer(inplace)
         if s.is_ket():
             s.data /= norm(s.data)
@@ -259,7 +348,8 @@ class state(lmap):
         if self.is_ket():
             return 1
         else:
-            return trace(dot(self.data, self.data))
+            # rho is hermitian so purity should be real
+            return trace(dot(self.data, self.data)) # .real
 
 
     def to_ket(self, inplace=False):
@@ -276,7 +366,7 @@ class state(lmap):
             if abs(s.purity() - 1) > tol:
                 raise ValueError('The state is not pure, and thus cannot be represented by a ket vector.')
 
-            d, v = eigsort(s.data)
+            d, v = eighsort(s.data)
             s.data = v[:, [0]]  # corresponds to the highest eigenvalue, i.e. 1
             s.fix_phase(inplace = True)  # clean up global phase
             s.dim = (s.dim[0], (1,))
@@ -291,7 +381,7 @@ class state(lmap):
         (q.data) is guaranteed to be a state operator.
         """
         # Ville Bergholm 2009-2010
-
+        # slight inefficiency when self.is_ket() and inplace==False: the ket data is copied for no reason
         s = self._inplacer(inplace)
         if s.is_ket():
             s.data = np.outer(s.data, s.data.conj())
@@ -306,12 +396,13 @@ class state(lmap):
         Returns the trace of the state operator of quantum state s.
         For a pure state this is equal to the squared norm of the state vector.
         """
-        # Ville Bergholm 2008
+        # Ville Bergholm 2008-2012
 
         if self.is_ket():
-            return vdot(self.data, self.data)
+            # squared norm, thus always real
+            return vdot(self.data, self.data).real
         else:
-            return trace(self.data)
+            return trace(self.data)  # .real
 
 
     def ptrace(self, sys, inplace=False):
@@ -345,9 +436,9 @@ class state(lmap):
             # build the index "stencil"
             inds = array([0])
             for k in range(n):
-                if k == j:
-                    continue
-                inds = tensorsum(inds, r_[0 : muls[k] * d[k] : muls[k]])
+                if k != j:
+                    inds = tensorsum(inds, r_[0 : muls[k] * d[k] : muls[k]])
+                    # np.arange(d[k]) * muls[k]
 
             stride = muls[j] # stride for moving the stencil while summing
             temp = len(inds)
@@ -368,8 +459,6 @@ class state(lmap):
 
 
 
-
-
     def ptranspose(self, sys, inplace=False):
         """Partial transpose.
 
@@ -435,7 +524,7 @@ class state(lmap):
         else:
             # state operator
             x = trace(dot(A, self.data))
-        return x.real # Hermitian observable
+        return x.real # .real for a Hermitian observable and valid state
 
 
     def var(self, A):
@@ -461,18 +550,20 @@ class state(lmap):
             temp = self.data.ravel() # into 1D array
             return (temp * temp.conj()).real  # == np.absolute(self.data) ** 2
         else:
-            return diag(self.data).real
+            return diag(self.data).real # .real
 
 
     def projector(self):
         """Projection operator defined by the state.
 
         Returns the projection operator P defined by the state.
-        TODO remove?        """
-        # Ville Bergholm 2009-2010
+        """
+        # Ville Bergholm 2009-2014
+        if abs(self.purity() - 1) > tol:
+            raise ValueError('The state is not pure, and thus does not correspond to a projector.')
 
         s = self.to_op()
-        return lmap(s.data, s.dim)
+        return lmap(s)
 
 
     def u_propagate(self, U):
@@ -498,7 +589,7 @@ class state(lmap):
             raise TypeError('States can only be propagated using lmaps and arrays.')
 
 
-    def propagate(self, G, t, out_func=lambda x, h: x, **kwargs):
+    def propagate(self, G, t, out_func=lambda x, h: deepcopy(x), **kwargs):
         r"""Propagate the state continuously in time.
         ::
 
@@ -642,7 +733,7 @@ class state(lmap):
 
             # do we want the initial state too? (the integrator can't handle t=0!)
             if t[0] == 0:
-                out.append(out_func(deepcopy(s), F(0)))
+                out.append(out_func(s, F(0)))
                 t = t[1:]
 
             # ODE solver default parameters
@@ -660,7 +751,7 @@ class state(lmap):
                 if not r.successful():
                     raise RuntimeError('ODE integrator failed.')
                 s.data = inv_vec(r.y, dim)
-                out.append(out_func(deepcopy(s), F(k)))
+                out.append(out_func(s, F(k)))
 
         else:
             # time independent case
@@ -725,6 +816,7 @@ class state(lmap):
     def measure(self, M=None, do='R'):
         r"""Quantum measurement.
         ::
+
           (p, res, c)
             = measure()                 measure the entire system projectively
             = measure((1, 4))           measure subsystems 1 and 4 projectively
@@ -920,13 +1012,13 @@ class state(lmap):
             if r.is_ket():
                 return abs(vdot(self.data, r.data))
             else:
-                return sqrt(vdot(self.data, dot(r.data, self.data)).real)
+                return sqrt(vdot(self.data, dot(r.data, self.data)).real)  # .real
         else:
             if r.is_ket():
-                return sqrt(vdot(r.data, dot(self.data, r.data)).real)
+                return sqrt(vdot(r.data, dot(self.data, r.data)).real)  # .real
             else:
                 temp = sp.linalg.sqrtm(self.data)
-                return trace(sp.linalg.sqrtm(dot(dot(temp, r.data), temp))).real
+                return trace(sp.linalg.sqrtm(dot(dot(temp, r.data), temp))).real  # .real
 
 
     def trace_dist(self, r):
@@ -943,11 +1035,12 @@ class state(lmap):
         if not isinstance(r, state):
             raise TypeError('Not a state.')
 
-        r = r.to_op()
-        s = self.to_op()
-        # TODO could use original data
-        A = r.data - s.data
-        return 0.5 * sum(abs(np.linalg.eigvalsh(A)))
+        # avoid copying state ops since we just do read-only stuff here
+        S = self.to_op() if self.is_ket() else self
+        R = r.to_op() if r.is_ket() else r
+
+        A = R.data - S.data
+        return 0.5 * sum(abs(eigvalsh(A)))
         #return 0.5*trace(sqrtm(A'*A))
 
 
@@ -1014,27 +1107,37 @@ class state(lmap):
             return s, u, vh.transpose()
 
 
-    def entropy(self, sys=None):
-        r"""Von Neumann entropy of the state.
 
-        If sys == None, returns the entropy of the state:
+    def entropy(self, sys=None, alpha=1):
+        r"""Von Neumann or Renyi entropy of the state.
+
+        Returns the Renyi entropy of order :math:`\alpha`,
+        :math:`S_\alpha(\rho) = \frac{1}{1-\alpha} \log_2 \mathrm{Tr}(\rho^\alpha)`.
+        
+        When :math:`\alpha = 1`, this coincides with the von Neumann entropy
         :math:`S(\rho) = -\mathrm{Tr}(\rho \log_2(\rho))`.
 
-        If a vector of subsystem indices sys is given, returns the
+        If sys == None, returns the entropy of the state.
+        If sys is a vector of subsystem indices, returns the
         entropy of entanglement of the state wrt. the partitioning
-        defined by sys.
-
-        Entropy of entanglement is only defined for pure states.
+        defined by sys. Entropy of entanglement is only defined for pure states.
         """
-        # Ville Bergholm 2009-2010
+        # Ville Bergholm 2009-2012
 
         if sys != None:
-            s = ptrace(to_ket(s), sys) # partial trace over one partition
-
-        if self.is_ket():
-            return 0
+            s = self.to_ket().ptrace(sys) # partial trace over one partition
         else:
-            p = np.linalg.eigvalsh(self.data)
+            s = self
+
+        if s.is_ket():
+            return 0
+
+        p = eigvalsh(s.data)
+        if alpha != 1:
+            # Rényi entropy
+            return np.log2(sum(p ** alpha)) / (1 - alpha)
+        else:
+            # Von Neumann entropy
             p[p == 0] = 1   # avoid trouble with the logarithm
             return -dot(p, np.log2(p))
 
@@ -1068,7 +1171,7 @@ class state(lmap):
             # pure state
             n = len(dim)
             rho_A = self.ptrace(self.invert_selection(sys)) # trace over everything but sys
-            C = 2 * sqrt(real(det(rho_A.data))) # = sqrt(2*(1-real(trace(temp*temp))))
+            C = 2 * sqrt(det(rho_A.data).real) # = sqrt(2*(1-real(trace(temp*temp)))), .real
             return
 
         # concurrence between two qubits
@@ -1092,10 +1195,10 @@ class state(lmap):
             temp = dot(p, X)  # X.conj() == X so this works
             temp = dot(temp, temp.conj())  # == p * X * conj(p) * X
             if abs(purity(self) - 1) > tol:
-                L = sqrt(sort(np.linalg.eigvals(temp).real)[::-1]).real
+                L = sqrt(sort(np.linalg.eigvals(temp).real)[::-1]).real  # .real?
                 C = max(0, L[1] -L[2] -L[3] -L[4])
             else:
-                C = sqrt(trace(temp).real) # same formula as for state vecs
+                C = sqrt(trace(temp).real) # same formula as for state vecs, .real?
 
 
     def negativity(self, sys):
@@ -1109,11 +1212,11 @@ class state(lmap):
         .. [Peres] A.Peres, "Separability Criterion for Density Matrices", PRL 77, 1413 (1996).
         .. [Horodecki1] M.Horodecki et al., "Separability of Mixed States: Necessary and Sufficient Conditions", Physics Letters A 223, 1-8 (1996).
         """
-        # Ville Bergholm 2008
+        # Ville Bergholm 2008-2014
 
         s = self.ptranspose(sys)  # partial transpose the state
         x = sp.linalg.svdvals(s.data)  # singular values
-        return (sum(sqrt(x)) - 1) / 2
+        return (sum(x) - 1) / 2
 
 
     def lognegativity(self, sys):
@@ -1142,34 +1245,32 @@ class state(lmap):
         .. [MW] D.A. Meyer and N.R. Wallach, "Global entanglement in multiparticle systems", J. Math. Phys. 43, 4273 (2002).
         """
         # Jacob D. Biamonte 2008
-        # Ville Bergholm 2008-2010
+        # Ville Bergholm 2008-2014
+
+        import itertools
 
         dim = self.dims()
         n = self.subsystems()
 
-        def nchoosek(n, k):
-            """List of k-combination lists of range(n)."""
-            # FIXME probably horribly inefficient, but sp.comb doesn't do this.
-            # power set of range(n)
-            temp = [[i for i in range(n) if x & (1 << i)] for x in range(2**n)]
-            # subset of elements with the correct length
-            return [i for i in temp if len(i) == k]
-
-        S = nchoosek(n, m)  # all m-combinations of n subsystems
+        if m < 1 or m > n-1:
+            raise ValueError('Partition size must be between 1 and n-1.')
 
         D = min(dim) # FIXME correct for arbitrary combinations of qudits??
-        C = (D**m / (D**m - 1)) / sp.comb(n, m)  # normalization
+        N = sp.misc.comb(n, m, exact=True)
+        C = (D**m / (D**m - 1)) / N  # normalization
 
-        Q = []
-        all_systems = set(range(n))
-        for sys in S:
-            temp = self.ptrace(all_systems.difference(sys))  # trace over everything except S_k
-            # NOTE: For pure states, tr(\rho_S^2) == tr(\rho_{\bar{S}}^2),
-            # so for them we could just use self.ptrace(sys) here.
-            Q.append(C * (1 - trace(np.linalg.matrix_power(temp.data, 2))))
+        Q = empty((N,))
+        # Loop over all m-combinations of n subsystems, trace over everything except them.
+        # reversed() fixes the order since we are actually looping over the complements.
+        for k, sys in enumerate(reversed(list(itertools.combinations(xrange(n), n-m)))):
+            temp = self.ptrace(sys)  # trace over everything except S_k
+            # NOTE: For pure states, tr(\rho_S^2) == tr(\rho_{\bar{S}}^2)
+            temp = 1 - trace(np.linalg.matrix_power(temp.data, 2))
+            Q[k] = C * temp.real
         return Q
 
 
+
     def locc_convertible(self, t, sys):
         """LOCC convertibility of states.
 
@@ -1192,7 +1293,7 @@ class state(lmap):
 
         s.ptrace(sys, inplace = True)
         t.ptrace(sys, inplace = True)
-        return majorize(np.linalg.eigvalsh(s.data), np.linalg.eigvalsh(t.data))
+        return majorize(eigvalsh(s.data), eigvalsh(t.data))
 
 
 
@@ -1268,10 +1369,11 @@ class state(lmap):
 
             c = phases(self.data)  # use phases as colors
             ax = fig.add_subplot(111, projection = '3d')
+            ax.view_init(40, -115)
 
             width = 0.6  # bar width
             temp = np.arange(-width/2, N-1) # center the labels
-            x, y = meshgrid(temp, temp)
+            x, y = meshgrid(temp, temp[::-1])
             x = x.ravel()
             y = y.ravel()
             z = abs(self.data.ravel())
@@ -1279,7 +1381,7 @@ class state(lmap):
             ax.bar3d(x, y, zeros(x.shape), dx, dx, z, color='b')
 
             # now the colors
-            pcol = ax.get_children()[2]  # poly3Dcollection
+            pcol = ax.get_children()[3]  # FIXME we need a robust way of getting the poly3Dcollection
             pcol.set_norm(nn)
             pcol.set_cmap(cm.hsv)
             pcol.set_array(kron(c.ravel(), (1,)*6))  # six faces per bar
@@ -1298,7 +1400,7 @@ class state(lmap):
             #ax.set_xticks(ticks)
             ax.set_xticklabels(ticklabels)
             #ax.set_yticks(ticks)
-            ax.set_yticklabels(ticklabels)
+            ax.set_yticklabels(ticklabels[::-1])
             #ax.set_alpha(0.8)
             # TODO ticks, ticklabels, alpha
 
@@ -1521,7 +1623,7 @@ class state(lmap):
         r = state(randn(30) + 1j*randn(30), [5, 6]).normalize()
         #x = r.schmidt((0,)) ** 2  # FIXME crashes ipython!
         temp = r.ptrace([1])
-        y, dummy = eigsort(temp.data)
+        y, dummy = eighsort(temp.data)
         #assert_o(norm(x-y), 0, tol)
 
 
